@@ -1,6 +1,6 @@
 /* Support for printing Fortran types for GDB, the GNU debugger.
 
-   Copyright (C) 1986-2022 Free Software Foundation, Inc.
+   Copyright (C) 1986-2024 Free Software Foundation, Inc.
 
    Contributed by Motorola.  Adapted from the C version by Farooq Butt
    (fmbutt@engage.sps.mot.com).
@@ -64,19 +64,19 @@ f_language::print_type (struct type *type, const char *varstring,
 	      || code == TYPE_CODE_ARRAY
 	      || ((code == TYPE_CODE_PTR
 		   || code == TYPE_CODE_REF)
-		  && (TYPE_TARGET_TYPE (type)->code () == TYPE_CODE_FUNC
-		      || (TYPE_TARGET_TYPE (type)->code ()
+		  && (type->target_type ()->code () == TYPE_CODE_FUNC
+		      || (type->target_type ()->code ()
 			  == TYPE_CODE_METHOD)
-		      || (TYPE_TARGET_TYPE (type)->code ()
+		      || (type->target_type ()->code ()
 			  == TYPE_CODE_ARRAY))))))
-    fputs_filtered (" ", stream);
+    gdb_puts (" ", stream);
   f_type_print_varspec_prefix (type, stream, show, 0);
 
   if (varstring != NULL)
     {
       int demangled_args;
 
-      fputs_filtered (varstring, stream);
+      gdb_puts (varstring, stream);
 
       /* For demangled function names, we have the arglist as part of the name,
 	 so don't print an additional pair of ()'s.  */
@@ -105,17 +105,17 @@ f_language::f_type_print_varspec_prefix (struct type *type,
   switch (type->code ())
     {
     case TYPE_CODE_PTR:
-      f_type_print_varspec_prefix (TYPE_TARGET_TYPE (type), stream, 0, 1);
+      f_type_print_varspec_prefix (type->target_type (), stream, 0, 1);
       break;
 
     case TYPE_CODE_FUNC:
-      f_type_print_varspec_prefix (TYPE_TARGET_TYPE (type), stream, 0, 0);
+      f_type_print_varspec_prefix (type->target_type (), stream, 0, 0);
       if (passed_a_ptr)
-	fprintf_filtered (stream, "(");
+	gdb_printf (stream, "(");
       break;
 
     case TYPE_CODE_ARRAY:
-      f_type_print_varspec_prefix (TYPE_TARGET_TYPE (type), stream, 0, 0);
+      f_type_print_varspec_prefix (type->target_type (), stream, 0, 0);
       break;
 
     case TYPE_CODE_UNDEF:
@@ -169,79 +169,79 @@ f_language::f_type_print_varspec_suffix (struct type *type,
       arrayprint_recurse_level++;
 
       if (arrayprint_recurse_level == 1)
-	fprintf_filtered (stream, "(");
+	gdb_printf (stream, "(");
 
       if (type_not_associated (type))
 	print_rank_only = true;
       else if (type_not_allocated (type))
 	print_rank_only = true;
       else if ((TYPE_ASSOCIATED_PROP (type)
-		&& PROP_CONST != TYPE_ASSOCIATED_PROP (type)->kind ())
+		&& !TYPE_ASSOCIATED_PROP (type)->is_constant ())
 	       || (TYPE_ALLOCATED_PROP (type)
-		   && PROP_CONST != TYPE_ALLOCATED_PROP (type)->kind ())
+		   && !TYPE_ALLOCATED_PROP (type)->is_constant ())
 	       || (TYPE_DATA_LOCATION (type)
-		   && PROP_CONST != TYPE_DATA_LOCATION (type)->kind ()))
+		   && !TYPE_DATA_LOCATION (type)->is_constant ()))
 	{
 	  /* This case exist when we ptype a typename which has the dynamic
 	     properties but cannot be resolved as there is no object.  */
 	  print_rank_only = true;
 	}
 
-      if (TYPE_TARGET_TYPE (type)->code () == TYPE_CODE_ARRAY)
-	f_type_print_varspec_suffix (TYPE_TARGET_TYPE (type), stream, 0,
+      if (type->target_type ()->code () == TYPE_CODE_ARRAY)
+	f_type_print_varspec_suffix (type->target_type (), stream, 0,
 				     0, 0, arrayprint_recurse_level,
 				     print_rank_only);
 
       if (print_rank_only)
-	fprintf_filtered (stream, ":");
+	gdb_printf (stream, ":");
       else
 	{
 	  LONGEST lower_bound = f77_get_lowerbound (type);
 	  if (lower_bound != 1)	/* Not the default.  */
-	    fprintf_filtered (stream, "%s:", plongest (lower_bound));
+	    gdb_printf (stream, "%s:", plongest (lower_bound));
 
 	  /* Make sure that, if we have an assumed size array, we
 	       print out a warning and print the upperbound as '*'.  */
 
 	  if (type->bounds ()->high.kind () == PROP_UNDEFINED)
-	    fprintf_filtered (stream, "*");
+	    gdb_printf (stream, "*");
 	  else
 	    {
 	      LONGEST upper_bound = f77_get_upperbound (type);
 
-	      fputs_filtered (plongest (upper_bound), stream);
+	      gdb_puts (plongest (upper_bound), stream);
 	    }
 	}
 
-      if (TYPE_TARGET_TYPE (type)->code () != TYPE_CODE_ARRAY)
-	f_type_print_varspec_suffix (TYPE_TARGET_TYPE (type), stream, 0,
+      if (type->target_type ()->code () != TYPE_CODE_ARRAY)
+	f_type_print_varspec_suffix (type->target_type (), stream, 0,
 				     0, 0, arrayprint_recurse_level,
 				     print_rank_only);
 
       if (arrayprint_recurse_level == 1)
-	fprintf_filtered (stream, ")");
+	gdb_printf (stream, ")");
       else
-	fprintf_filtered (stream, ",");
+	gdb_printf (stream, ",");
       arrayprint_recurse_level--;
       break;
 
     case TYPE_CODE_PTR:
     case TYPE_CODE_REF:
-      f_type_print_varspec_suffix (TYPE_TARGET_TYPE (type), stream, 0, 1, 0,
+      f_type_print_varspec_suffix (type->target_type (), stream, 0, 1, 0,
 				   arrayprint_recurse_level, false);
-      fprintf_filtered (stream, " )");
+      gdb_printf (stream, " )");
       break;
 
     case TYPE_CODE_FUNC:
       {
 	int i, nfields = type->num_fields ();
 
-	f_type_print_varspec_suffix (TYPE_TARGET_TYPE (type), stream, 0,
+	f_type_print_varspec_suffix (type->target_type (), stream, 0,
 				     passed_a_ptr, 0,
 				     arrayprint_recurse_level, false);
 	if (passed_a_ptr)
-	  fprintf_filtered (stream, ") ");
-	fprintf_filtered (stream, "(");
+	  gdb_printf (stream, ") ");
+	gdb_printf (stream, "(");
 	if (nfields == 0 && type->is_prototyped ())
 	  print_type (builtin_f_type (type->arch ())->builtin_void,
 		      "", stream, -1, 0, 0);
@@ -250,12 +250,12 @@ f_language::f_type_print_varspec_suffix (struct type *type,
 	    {
 	      if (i > 0)
 		{
-		  fputs_filtered (", ", stream);
+		  gdb_puts (", ", stream);
 		  stream->wrap_here (4);
 		}
 	      print_type (type->field (i).type (), "", stream, -1, 0, 0);
 	    }
-	fprintf_filtered (stream, ")");
+	gdb_printf (stream, ")");
       }
       break;
 
@@ -292,7 +292,7 @@ f_language::f_type_print_derivation_info (struct type *type,
   const int i = 0;
 
   if (TYPE_N_BASECLASSES (type) > 0)
-    fprintf_filtered (stream, ", extends(%s) ::", TYPE_BASECLASS (type, i)->name ());
+    gdb_printf (stream, ", extends(%s) ::", TYPE_BASECLASS (type, i)->name ());
 }
 
 /* See f-lang.h.  */
@@ -321,9 +321,9 @@ f_language::f_type_print_base (struct type *type, struct ui_file *stream,
       if (type->code () == TYPE_CODE_UNION)
 	prefix = "Type, C_Union :: ";
       else if (type->code () == TYPE_CODE_STRUCT
-               || type->code () == TYPE_CODE_NAMELIST)
+	       || type->code () == TYPE_CODE_NAMELIST)
 	prefix = "Type ";
-      fprintf_filtered (stream, "%*s%s%s", level, "", prefix, type->name ());
+      gdb_printf (stream, "%*s%s%s", level, "", prefix, type->name ());
       return;
     }
 
@@ -333,47 +333,47 @@ f_language::f_type_print_base (struct type *type, struct ui_file *stream,
   switch (type->code ())
     {
     case TYPE_CODE_TYPEDEF:
-      f_type_print_base (TYPE_TARGET_TYPE (type), stream, 0, level);
+      f_type_print_base (type->target_type (), stream, 0, level);
       break;
 
     case TYPE_CODE_ARRAY:
-      f_type_print_base (TYPE_TARGET_TYPE (type), stream, show, level);
+      f_type_print_base (type->target_type (), stream, show, level);
       break;
     case TYPE_CODE_FUNC:
-      if (TYPE_TARGET_TYPE (type) == NULL)
+      if (type->target_type () == NULL)
 	type_print_unknown_return_type (stream);
       else
-	f_type_print_base (TYPE_TARGET_TYPE (type), stream, show, level);
+	f_type_print_base (type->target_type (), stream, show, level);
       break;
 
     case TYPE_CODE_PTR:
-      fprintf_filtered (stream, "%*sPTR TO -> ( ", level, "");
-      f_type_print_base (TYPE_TARGET_TYPE (type), stream, show, 0);
+      gdb_printf (stream, "%*sPTR TO -> ( ", level, "");
+      f_type_print_base (type->target_type (), stream, show, 0);
       break;
 
     case TYPE_CODE_REF:
-      fprintf_filtered (stream, "%*sREF TO -> ( ", level, "");
-      f_type_print_base (TYPE_TARGET_TYPE (type), stream, show, 0);
+      gdb_printf (stream, "%*sREF TO -> ( ", level, "");
+      f_type_print_base (type->target_type (), stream, show, 0);
       break;
 
     case TYPE_CODE_VOID:
       {
 	struct type *void_type = builtin_f_type (type->arch ())->builtin_void;
-	fprintf_filtered (stream, "%*s%s", level, "", void_type->name ());
+	gdb_printf (stream, "%*s%s", level, "", void_type->name ());
       }
       break;
 
     case TYPE_CODE_UNDEF:
-      fprintf_filtered (stream, "%*sstruct <unknown>", level, "");
+      gdb_printf (stream, "%*sstruct <unknown>", level, "");
       break;
 
     case TYPE_CODE_ERROR:
-      fprintf_filtered (stream, "%*s%s", level, "", TYPE_ERROR_NAME (type));
+      gdb_printf (stream, "%*s%s", level, "", TYPE_ERROR_NAME (type));
       break;
 
     case TYPE_CODE_RANGE:
       /* This should not occur.  */
-      fprintf_filtered (stream, "%*s<range type>", level, "");
+      gdb_printf (stream, "%*s<range type>", level, "");
       break;
 
     case TYPE_CODE_CHAR:
@@ -383,7 +383,7 @@ f_language::f_type_print_base (struct type *type, struct ui_file *stream,
 	 C-oriented, we must change these to "character" from "char".  */
 
       if (strcmp (type->name (), "char") == 0)
-	fprintf_filtered (stream, "%*scharacter", level, "");
+	gdb_printf (stream, "%*scharacter", level, "");
       else
 	goto default_case;
       break;
@@ -395,54 +395,54 @@ f_language::f_type_print_base (struct type *type, struct ui_file *stream,
 	 asked to print the type of a value with a dynamic type then the
 	 bounds will not have been resolved.  */
 
-      if (type->bounds ()->high.kind () == PROP_CONST)
+      if (type->bounds ()->high.is_constant ())
 	{
 	  LONGEST upper_bound = f77_get_upperbound (type);
 
-	  fprintf_filtered (stream, "character*%s", pulongest (upper_bound));
+	  gdb_printf (stream, "character*%s", pulongest (upper_bound));
 	}
       else
-	fprintf_filtered (stream, "%*scharacter*(*)", level, "");
+	gdb_printf (stream, "%*scharacter*(*)", level, "");
       break;
 
     case TYPE_CODE_STRUCT:
     case TYPE_CODE_UNION:
     case TYPE_CODE_NAMELIST:
       if (type->code () == TYPE_CODE_UNION)
-	fprintf_filtered (stream, "%*sType, C_Union ::", level, "");
+	gdb_printf (stream, "%*sType, C_Union ::", level, "");
       else
-	fprintf_filtered (stream, "%*sType", level, "");
+	gdb_printf (stream, "%*sType", level, "");
 
       if (show > 0)
 	f_type_print_derivation_info (type, stream);
 
-      fputs_filtered (" ", stream);
+      gdb_puts (" ", stream);
 
-      fputs_filtered (type->name (), stream);
+      gdb_puts (type->name (), stream);
 
       /* According to the definition,
 	 we only print structure elements in case show > 0.  */
       if (show > 0)
 	{
-	  fputs_filtered ("\n", stream);
+	  gdb_puts ("\n", stream);
 	  for (index = 0; index < type->num_fields (); index++)
 	    {
 	      f_type_print_base (type->field (index).type (), stream,
 				 show - 1, level + 4);
-	      fputs_filtered (" :: ", stream);
+	      gdb_puts (" :: ", stream);
 	      fputs_styled (type->field (index).name (),
 			    variable_name_style.style (), stream);
 	      f_type_print_varspec_suffix (type->field (index).type (),
 					   stream, show - 1, 0, 0, 0, false);
-	      fputs_filtered ("\n", stream);
+	      gdb_puts ("\n", stream);
 	    }
-	  fprintf_filtered (stream, "%*sEnd Type ", level, "");
-	  fputs_filtered (type->name (), stream);
+	  gdb_printf (stream, "%*sEnd Type ", level, "");
+	  gdb_puts (type->name (), stream);
 	}
       break;
 
     case TYPE_CODE_MODULE:
-      fprintf_filtered (stream, "%*smodule %s", level, "", type->name ());
+      gdb_printf (stream, "%*smodule %s", level, "", type->name ());
       break;
 
     default_case:
@@ -452,12 +452,12 @@ f_language::f_type_print_base (struct type *type, struct ui_file *stream,
 	 the type name is, as recorded in the type itself.  If there
 	 is no type name, then complain.  */
       if (type->name () != NULL)
-	fprintf_filtered (stream, "%*s%s", level, "", type->name ());
+	gdb_printf (stream, "%*s%s", level, "", type->name ());
       else
 	error (_("Invalid type code (%d) in symbol table."), type->code ());
       break;
     }
 
   if (TYPE_IS_ALLOCATABLE (type))
-    fprintf_filtered (stream, ", allocatable");
+    gdb_printf (stream, ", allocatable");
 }
