@@ -1,6 +1,6 @@
 /* The find command.
 
-   Copyright (C) 2008-2023 Free Software Foundation, Inc.
+   Copyright (C) 2008-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -110,7 +110,7 @@ parse_find_args (const char *args, ULONGEST *max_countp,
   v = parse_to_comma_and_eval (&s);
   start_addr = value_as_address (v);
 
-  struct type *start_addr_type = check_typedef (value_type (v));
+  struct type *start_addr_type = check_typedef (v->type ());
   struct gdbarch *gdbarch = start_addr_type->arch ();
 
   /* For targets with multiple address spaces.  */
@@ -178,7 +178,7 @@ parse_find_args (const char *args, ULONGEST *max_countp,
       s = skip_spaces (s);
 
       v = parse_to_comma_and_eval (&s);
-      t = value_type (v);
+      t = v->type ();
 
       if (gdbarch_address_space_from_type_flags_p (gdbarch))
 	{
@@ -186,7 +186,7 @@ parse_find_args (const char *args, ULONGEST *max_countp,
 	     about the address space requested.  We therefore need to adjust
 	     start_addr here and check that there weren't two contradicting
 	     address spaces requested.  */
-	  struct type *pattern_type = check_typedef (value_type (v));
+	  struct type *pattern_type = check_typedef (v->type ());
 
 	  addr_space_t_pattern = gdbarch_address_space_from_type_flags (
 					gdbarch,
@@ -224,7 +224,7 @@ parse_find_args (const char *args, ULONGEST *max_countp,
 	}
       else
 	{
-	  const gdb_byte *contents = value_contents (v).data ();
+	  const gdb_byte *contents = v->contents ().data ();
 	  pattern_buf.insert (pattern_buf.end (), contents,
 			      contents + t->length ());
 	}
