@@ -1,6 +1,6 @@
 /* GNU/Linux/x86-64 specific target description, for the remote server
    for GDB.
-   Copyright (C) 2017-2023 Free Software Foundation, Inc.
+   Copyright (C) 2017-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -104,8 +104,8 @@ i386_linux_read_description (uint64_t xcr0)
 static target_desc *amd64_tdescs[X86_TDESC_LAST] = { };
 static target_desc *x32_tdescs[X86_TDESC_LAST] = { };
 
-const struct target_desc *
-amd64_linux_read_description (uint64_t xcr0, bool is_x32)
+const target_desc *
+amd64_linux_read_description (uint64_t xcr0, bool is_x32, bool ssp_enabled)
 {
   enum x86_linux_tdesc idx = xcr0_to_tdesc_idx (xcr0, is_x32);
 
@@ -121,7 +121,8 @@ amd64_linux_read_description (uint64_t xcr0, bool is_x32)
 
   if (*tdesc == NULL)
     {
-      *tdesc = amd64_create_target_description (xcr0, is_x32, true, true);
+      *tdesc = amd64_create_target_description (xcr0, is_x32, true, true,
+						ssp_enabled);
 
       init_target_desc (*tdesc, amd64_expedite_regs);
     }
