@@ -1,6 +1,6 @@
 /* Python interface to btrace instruction history.
 
-   Copyright 2016-2023 Free Software Foundation, Inc.
+   Copyright 2016-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -45,7 +45,8 @@ struct btpy_list_object {
   /* Stride size.  */
   Py_ssize_t step;
 
-  /* Either &BTPY_CALL_TYPE or &RECPY_INSN_TYPE.  */
+  /* Either &recpy_func_type, &recpy_insn_type, &recpy_aux_type or
+     &recpy_gap_type.  */
   PyTypeObject* element_type;
 };
 
@@ -170,13 +171,10 @@ btpy_item_new (thread_info *tinfo, Py_ssize_t number)
     }
 
   const struct btrace_insn *insn = btrace_insn_get (&iter);
-
-  if (insn == nullptr)
-    return nullptr;
+  gdb_assert (insn != nullptr);
 
   if (insn->iclass == BTRACE_INSN_AUX)
-    return recpy_aux_new (iter.btinfo->aux_data[insn->aux_data_index].c_str (),
-			  number);
+    return recpy_aux_new (tinfo, RECORD_METHOD_BTRACE, number);
 
   return recpy_insn_new (tinfo, RECORD_METHOD_BTRACE, number);
 }
@@ -439,6 +437,48 @@ recpy_bt_func_next (PyObject *self, void *closure)
 			 RECORD_METHOD_BTRACE, func->next);
 }
 
+/* Implementation of Auxiliary.data [str] for btrace.  */
+
+PyObject *
+recpy_bt_aux_data (PyObject *self, void *closure)
+{
+  const btrace_insn *insn;
+  const recpy_element_object *obj;
+  thread_info *tinfo;
+  btrace_insn_iterator iter;
+
+  if (Py_TYPE (self) != &recpy_aux_type)
+    {
+      PyErr_Format (gdbpy_gdb_error, _("Must be a gdb.Auxiliary."));
+      return NULL;
+    }
+
+  obj = (const recpy_element_object *) self;
+  tinfo = obj->thread;
+
+  if (tinfo == NULL || btrace_is_empty (tinfo))
+    {
+      PyErr_Format (gdbpy_gdb_error, _("No such auxiliary object."));
+      return NULL;
+    }
+
+  if (btrace_find_insn_by_number (&iter, &tinfo->btrace, obj->number) == 0)
+    {
+      PyErr_Format (gdbpy_gdb_error, _("No such auxiliary object."));
+      return NULL;
+    }
+
+  insn = btrace_insn_get (&iter);
+  if (insn == NULL || insn->iclass != BTRACE_INSN_AUX)
+    {
+      PyErr_Format (gdbpy_gdb_error, _("Not a valid auxiliary object."));
+      return NULL;
+    }
+
+  return PyUnicode_FromString
+    (iter.btinfo->aux_data.at (insn->aux_data_index).c_str ());
+}
+
 /* Implementation of BtraceList.__len__ (self) -> int.  */
 
 static Py_ssize_t
@@ -455,8 +495,9 @@ btpy_list_length (PyObject *self)
 }
 
 /* Implementation of
-   BtraceList.__getitem__ (self, key) -> BtraceInstruction and
-   BtraceList.__getitem__ (self, key) -> BtraceFunctionCall.  */
+   BtraceList.__getitem__ (self, key) -> BtraceInstruction,
+   BtraceList.__getitem__ (self, key) -> BtraceFunctionCall,
+   BtraceList.__getitem__ (self, key) -> BtraceAuxiliary.  */
 
 static PyObject *
 btpy_list_item (PyObject *self, Py_ssize_t index)
@@ -470,10 +511,13 @@ btpy_list_item (PyObject *self, Py_ssize_t index)
 
   number = obj->first + (obj->step * index);
 
-  if (obj->element_type == &recpy_insn_type)
+  if (obj->element_type == &recpy_func_type)
+    return recpy_func_new (obj->thread, RECORD_METHOD_BTRACE, number);
+  else if (obj->element_type == &recpy_insn_type
+	   || obj->element_type == &recpy_aux_type)
     return btpy_item_new (obj->thread, number);
   else
-    return recpy_func_new (obj->thread, RECORD_METHOD_BTRACE, number);
+    return PyErr_Format (gdbpy_gdb_error, _("Not a valid BtraceList object."));
 }
 
 /* Implementation of BtraceList.__getitem__ (self, slice) -> BtraceList.  */
@@ -765,75 +809,73 @@ recpy_bt_function_call_history (PyObject *self, void *closure)
   return btpy_list_new (tinfo, first, last, 1, &recpy_func_type);
 }
 
-/* Calling the ptwrite listener.  Returns a pointer to the string that will be
-   printed or nullptr if nothing should be printed.  */
-gdb::unique_xmalloc_ptr<char>
-recpy_call_listener (const uint64_t *payload, const uint64_t *ip,
-		     const void *ptw_listener)
+/* Helper function that calls PTW_FILTER with PAYLOAD and IP as arguments.
+   Returns the string that will be printed, if there is a filter to call.  */
+static gdb::optional<std::string>
+recpy_call_filter (const uint64_t payload, const uint64_t ip,
+		   const void *ptw_filter)
 {
-  if ((PyObject *) ptw_listener == Py_None)
-    return nullptr;
-  else if ((PyObject *) ptw_listener == nullptr)
-    error (_("No valid ptwrite listener."));
+  gdb::optional<std::string> result;
 
-  /* As Python is started as a seperate thread, we need to
-     acquire the GIL to safely call the listener function.  */
-  PyGILState_STATE gstate = PyGILState_Ensure ();
+  gdb_assert (ptw_filter != nullptr);
+  if ((PyObject *) ptw_filter == Py_None)
+    return result;
 
-  PyObject *py_ip = Py_None;
-  PyObject *py_payload = PyLong_FromUnsignedLongLong (*payload);
-  Py_INCREF (Py_None);
+  gdbpy_enter enter_py;
 
-  if (ip != nullptr)
-    py_ip = PyLong_FromUnsignedLongLong (*ip);
+  gdbpy_ref<> py_payload (PyLong_FromUnsignedLongLong (payload));
 
-  PyObject *py_result = PyObject_CallFunctionObjArgs ((PyObject *) ptw_listener,
-						      py_payload, py_ip, NULL);
+  if (py_payload == nullptr)
+    return result;
 
-  if (PyErr_Occurred ())
+  gdbpy_ref<> py_ip;
+
+  if (ip == 0)
+    py_ip = gdbpy_ref<>::new_reference (Py_None);
+  else
+    py_ip = gdbpy_ref<>::new_reference (PyLong_FromUnsignedLongLong (ip));
+
+  if (py_ip == nullptr)
+    return result;
+
+  gdbpy_ref<> py_result (PyObject_CallFunctionObjArgs ((PyObject *) ptw_filter,
+							py_payload.get (),
+							py_ip.get (),
+							nullptr));
+
+  if (py_result == nullptr)
     {
       gdbpy_print_stack ();
-      Py_DECREF (py_ip);
-      Py_DECREF (py_payload);
-      Py_DECREF (py_result);
-      PyGILState_Release (gstate);
-      error (_("Error while executing Python code."));
+      gdbpy_error (_("Couldn't call the ptwrite filter."));
     }
 
-  Py_DECREF (py_ip);
-  Py_DECREF (py_payload);
-
+  /* Py_None is valid and results in no output.  */
   if (py_result == Py_None)
     {
-      Py_DECREF (py_result);
-      PyGILState_Release (gstate);
-      return nullptr;
+      result = "";
+      return result;
     }
 
-  gdb::unique_xmalloc_ptr<char> resultstring = gdbpy_obj_to_string (py_result);
+  gdb::unique_xmalloc_ptr<char> user_string
+    = gdbpy_obj_to_string (py_result.get ());
 
-  if (PyErr_Occurred ())
+  if (user_string == nullptr)
     {
       gdbpy_print_stack ();
-      Py_DECREF (py_result);
-      PyGILState_Release (gstate);
-      error (_("Error while executing Python code."));
+      gdbpy_error (_("The ptwrite filter didn't return a string."));
     }
+  else
+    result = user_string.get ();
 
-  if (py_result != nullptr)
-    Py_DECREF (py_result);
-  PyGILState_Release (gstate);
-
-  return resultstring;
+  return result;
 }
 
-/* Helper function returning the current ptwrite listener.  Returns nullptr
-   in case of errors.  */
+/* Helper function returning the current ptwrite filter.  */
 
-PyObject *
-get_ptwrite_listener ()
+static PyObject *
+get_ptwrite_filter ()
 {
-  PyObject *module = PyImport_ImportModule ("gdb.ptwrite");
+  gdbpy_ref<> module (PyImport_ImportModule ("gdb.ptwrite"));
 
   if (PyErr_Occurred ())
   {
@@ -841,32 +883,33 @@ get_ptwrite_listener ()
     return nullptr;
   }
 
-  PyObject *default_ptw_listener = PyObject_CallMethod (module,	"get_listener",
-							NULL);
-
-  Py_DECREF (module);
+  /* We need to keep the reference count.  */
+  gdbpy_ref<> ptw_filter (PyObject_CallMethod (module.get (), "get_filter",
+					       nullptr));
 
   if (PyErr_Occurred ())
-    gdbpy_print_stack ();
+    {
+      gdbpy_print_stack ();
+      gdbpy_error (_("Couldn't get the ptwrite filter."));
+    }
 
-  return default_ptw_listener;
+  return ptw_filter.get();
 }
 
-/* Helper function to initialize the ptwrite listener.  Returns nullptr in
-   case of errors.  */
+/* Used for registering any python ptwrite filter to the current thread.  A
+   pointer to this function is stored in the python extension interface.  */
 
-PyObject *
-recpy_initialize_listener (ptid_t inferior_ptid)
+void
+gdbpy_load_ptwrite_filter (const struct extension_language_defn *extlang,
+			   struct btrace_thread_info *btinfo)
 {
-  process_stratum_target *proc_target
-    = current_inferior ()->process_target ();
-  struct thread_info * const tinfo
-    = find_thread_ptid (proc_target, inferior_ptid);
+  gdb_assert (btinfo != nullptr);
 
-  tinfo->btrace.ptw_callback_fun = &recpy_call_listener;
-  tinfo->btrace.ptw_listener = get_ptwrite_listener ();
+  gdbpy_enter enter_py;
 
-  return (PyObject *) tinfo->btrace.ptw_listener;
+  btinfo->ptw_context = get_ptwrite_filter ();
+  if (btinfo->ptw_context != nullptr)
+    btinfo->ptw_callback_fun = &recpy_call_filter;
 }
 
 /* Implementation of BtraceRecord.goto (self, BtraceInstruction) -> None.  */
@@ -946,7 +989,7 @@ static PyMappingMethods btpy_list_mapping_methods =
 
 /* Sets up the btrace record API.  */
 
-int
+static int CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION
 gdbpy_initialize_btrace (void)
 {
   btpy_list_type.tp_new = PyType_GenericNew;
@@ -967,3 +1010,5 @@ gdbpy_initialize_btrace (void)
 
   return PyType_Ready (&btpy_list_type);
 }
+
+GDBPY_INITIALIZE_FILE (gdbpy_initialize_btrace);
