@@ -1,6 +1,6 @@
 /* Python interface to record targets.
 
-   Copyright 2016-2023 Free Software Foundation, Inc.
+   Copyright 2016-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -24,7 +24,6 @@
 #include "py-record-full.h"
 #include "target.h"
 #include "gdbthread.h"
-#include "gdbarch.h"
 
 /* Python Record type.  */
 
@@ -53,7 +52,7 @@ static PyTypeObject recpy_gap_type = {
 /* Python RecordAuxiliary type.  */
 
 PyTypeObject recpy_aux_type = {
-  PyVarObject_HEAD_INIT (NULL, 0)
+  PyVarObject_HEAD_INIT (nullptr, 0)
 };
 
 /* Python RecordGap object.  */
@@ -71,18 +70,6 @@ struct recpy_gap_object
   Py_ssize_t number;
 };
 
-/* Python RecordAuxiliary object.  */
-typedef struct
-{
-  PyObject_HEAD
-
-  /* Auxiliary data.  */
-  const char *data;
-
-  /* Element number.  */
-  Py_ssize_t number;
-} recpy_aux_object;
-
 /* Implementation of record.method.  */
 
 static PyObject *
@@ -128,7 +115,7 @@ recpy_goto (PyObject *self, PyObject *value)
   return PyErr_Format (PyExc_NotImplementedError, _("Not implemented."));
 }
 
-/* Implementation of record.clear_trace () -> None.  */
+/* Implementation of record.clear () -> None.  */
 
 static PyObject *
 recpy_clear (PyObject *self, PyObject *value)
@@ -422,8 +409,8 @@ recpy_element_hash (PyObject *self)
   return obj->number;
 }
 
-/* Implementation of operator == and != of RecordInstruction and
-   RecordFunctionSegment.  */
+/* Implementation of operator == and != of RecordInstruction,
+   RecordFunctionSegment and RecordAuxiliary.  */
 
 static PyObject *
 recpy_element_richcompare (PyObject *self, PyObject *other, int op)
@@ -514,38 +501,33 @@ recpy_gap_reason_string (PyObject *self, void *closure)
 /* Create a new gdb.Auxiliary object.  */
 
 PyObject *
-recpy_aux_new (const char *data, Py_ssize_t number)
+recpy_aux_new (thread_info *thread, enum record_method method,
+	       Py_ssize_t number)
 {
-  recpy_aux_object * const obj = PyObject_New (recpy_aux_object,
-					       &recpy_aux_type);
+  recpy_element_object * const obj = PyObject_New (recpy_element_object,
+						   &recpy_aux_type);
 
   if (obj == NULL)
    return NULL;
 
-  obj->data = data;
+  obj->thread = thread;
+  obj->method = method;
   obj->number = number;
 
   return (PyObject *) obj;
 }
 
-/* Implementation of Auxiliary.number [int].  */
-
-static PyObject *
-recpy_aux_number (PyObject *self, void *closure)
-{
-  const recpy_aux_object * const obj = (const recpy_aux_object *) self;
-
-  return gdb_py_object_from_longest (obj->number).release ();
-}
-
-/* Implementation of Auxiliary.data [str].  */
+/* Implementation of Auxiliary.data [buffer].  */
 
 static PyObject *
 recpy_aux_data (PyObject *self, void *closure)
 {
-  const recpy_aux_object * const obj = (const recpy_aux_object *) self;
+  const recpy_element_object * const obj = (recpy_element_object *) self;
 
-  return PyUnicode_FromString (obj->data);
+  if (obj->method == RECORD_METHOD_BTRACE)
+    return recpy_bt_aux_data (self, closure);
+
+  return PyErr_Format (PyExc_NotImplementedError, _("Not implemented."));
 }
 
 /* Record method list.  */
@@ -554,8 +536,8 @@ static PyMethodDef recpy_record_methods[] = {
   { "goto", recpy_goto, METH_VARARGS,
     "goto (instruction|function_call) -> None.\n\
 Rewind to given location."},
-  { "clear_trace", recpy_clear, METH_VARARGS,
-    "clear_trace () -> None.\n\
+  { "clear", recpy_clear, METH_VARARGS,
+    "clear () -> None.\n\
 Clears the trace."},
   { NULL }
 };
@@ -619,14 +601,14 @@ static gdb_PyGetSetDef recpy_gap_getset[] = {
 /* RecordAuxiliary member list.  */
 
 static gdb_PyGetSetDef recpy_aux_getset[] = {
-  { "number", recpy_aux_number, NULL, "element number", NULL},
-  { "data", recpy_aux_data, NULL, "data", NULL},
-  { NULL }
+  { "number", recpy_element_number, nullptr, "element number", nullptr},
+  { "data", recpy_aux_data, nullptr, "data", nullptr},
+  { nullptr }
 };
 
 /* Sets up the record API in the gdb module.  */
 
-int
+static int CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION
 gdbpy_initialize_record (void)
 {
   recpy_record_type.tp_new = PyType_GenericNew;
@@ -665,10 +647,12 @@ gdbpy_initialize_record (void)
 
   recpy_aux_type.tp_new = PyType_GenericNew;
   recpy_aux_type.tp_flags = Py_TPFLAGS_DEFAULT;
-  recpy_aux_type.tp_basicsize = sizeof (recpy_aux_object);
+  recpy_aux_type.tp_basicsize = sizeof (recpy_element_object);
   recpy_aux_type.tp_name = "gdb.RecordAuxiliary";
   recpy_aux_type.tp_doc = "GDB recorded auxiliary object";
   recpy_aux_type.tp_getset = recpy_aux_getset;
+  recpy_aux_type.tp_richcompare = recpy_element_richcompare;
+  recpy_aux_type.tp_hash = recpy_element_hash;
 
   if (PyType_Ready (&recpy_record_type) < 0
       || PyType_Ready (&recpy_insn_type) < 0
@@ -739,31 +723,4 @@ gdbpy_stop_recording (PyObject *self, PyObject *args)
   Py_RETURN_NONE;
 }
 
-/* Used for registering the default ptwrite listener to the current thread.  A
-   pointer to this function is stored in the python extension interface.  */
-
-enum ext_lang_bt_status
-gdbpy_load_ptwrite_listener (const struct extension_language_defn *extlang,
-			     ptid_t inferior_ptid)
-{
-  struct gdbarch *gdbarch = NULL;
-
-  if (!gdb_python_initialized)
-    return EXT_LANG_BT_NO_FILTERS;
-
-  try
-    {
-      gdbarch = target_gdbarch ();
-    }
-  catch (const gdb_exception &except)
-    {
-      /* Let gdb try to print the stack trace.  */
-      return EXT_LANG_BT_NO_FILTERS;
-    }
-
-  gdbpy_enter enter_py (gdbarch, current_language);
-
-  recpy_initialize_listener (inferior_ptid);
-
-  return EXT_LANG_BT_OK;
-}
+GDBPY_INITIALIZE_FILE (gdbpy_initialize_record);
