@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2013-2021, Intel Corporation
+ * Copyright (c) 2013-2023, Intel Corporation
+ * SPDX-License-Identifier: BSD-3-Clause
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -36,17 +37,18 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <limits.h>
 
 #if defined(_MSC_VER) && (_MSC_VER < 1900)
 #  define snprintf _snprintf_c
 #endif
 
 
-static const char *pt_suffix = ".pt";
-static const char *exp_suffix = ".exp";
+static const char pt_suffix[] = ".pt";
+static const char exp_suffix[] = ".exp";
 
 #if defined(FEATURE_SIDEBAND)
-static const char *sb_suffix = ".sb";
+static const char sb_suffix[] = ".sb";
 #endif
 
 enum {
@@ -58,14 +60,16 @@ enum {
 static void sb_rename_file(struct sb_file *sb)
 {
 	char filename[FILENAME_MAX];
+	int len;
 
 	/* We encode the configuration in the sideband filename. */
 	switch (sb->format) {
 	case sbf_raw:
-		strncpy(filename, sb->name, sizeof(filename) - 1);
-
-		/* Make sure @filename is terminated. */
-		filename[sizeof(filename) - 1] = 0;
+		len = snprintf(filename, sizeof(filename), "%s", sb->name);
+		if ((len < 0) || (sizeof(filename) <= (size_t) len)) {
+			fprintf(stderr, "error renaming %s.\n", sb->name);
+			return;
+		}
 		break;
 
 #if defined(FEATURE_PEVENT)
@@ -89,7 +93,7 @@ static void sb_rename_file(struct sb_file *sb)
 
 		ext_len = (size_t) printed;
 
-		suffix_len = strnlen(sb_suffix, sizeof(filename));
+		suffix_len = sizeof(sb_suffix) - 1;
 		base_len = strnlen(sb->name, sizeof(filename));
 		if (base_len < suffix_len) {
 			fprintf(stderr, "error renaming %s.\n", sb->name);
@@ -105,12 +109,15 @@ static void sb_rename_file(struct sb_file *sb)
 			return;
 		}
 
-		strncpy(filename, sb->name, base_len);
+		if (INT_MAX < base_len) {
+			fprintf(stderr, "error renaming %s.\n", sb->name);
+			return;
+		}
 
-		printed = snprintf(filename + base_len,
-				   sizeof(filename) - base_len, "%s%s",
-				   extension, sb_suffix);
-		if (printed < 0) {
+		printed = snprintf(filename, sizeof(filename), "%.*s%s%s",
+				   (int) base_len, sb->name, extension,
+				   sb_suffix);
+		if ((printed < 0) || (sizeof(filename) <= (size_t) printed)) {
 			fprintf(stderr, "error renaming %s.\n", sb->name);
 			return;
 		}
@@ -126,7 +133,7 @@ static void sb_rename_file(struct sb_file *sb)
 	}
 
 	/* Print the name of the sideband file for test.bash. */
-	printf("%s\n", filename);
+	printf("%s ", filename);
 }
 
 #endif /* defined(FEATURE_SIDEBAND) */
@@ -175,8 +182,9 @@ static void p_free(struct parser *p)
  */
 static struct parser *p_alloc(const char *pttfile, const struct pt_config *conf)
 {
-	size_t n;
+	size_t n, size;
 	struct parser *p;
+	int len;
 
 	if (!conf)
 		return NULL;
@@ -192,14 +200,19 @@ static struct parser *p_alloc(const char *pttfile, const struct pt_config *conf)
 	if (!p->y)
 		goto error;
 
-	n = strlen(p->y->fileroot) + 1;
+	n = strnlen(p->y->fileroot, FILENAME_MAX - sizeof(pt_suffix));
+	if ((FILENAME_MAX - sizeof(pt_suffix)) <= n)
+		goto error;
 
-	p->ptfilename = malloc(n+strlen(pt_suffix));
+	size = n + sizeof(pt_suffix);
+
+	p->ptfilename = malloc(size);
 	if (!p->ptfilename)
 		goto error;
 
-	strcpy(p->ptfilename, p->y->fileroot);
-	strcat(p->ptfilename, pt_suffix);
+	len = snprintf(p->ptfilename, size, "%s%s", p->y->fileroot, pt_suffix);
+	if ((len < 0) || ((size_t) len != (size - 1)))
+		goto error;
 
 	p->pd = pd_alloc(pd_len);
 	if (!p->pd)
@@ -224,68 +237,50 @@ error:
 }
 
 /* Generates an .exp filename following the scheme:
- *	<fileroot>[-<extra>].exp
+ *	<fileroot>[-<extra>][-cpu_<f>_<m>[_<s>]].exp
  */
 static char *expfilename(struct parser *p, const char *extra)
 {
-	char *filename;
-	/* reserve enough space to hold the string
-	 *   "-cpu_fffff_mmm_sss" + 1 for the trailing null character.
-	 */
-	char cpu_suffix[19];
-	size_t n;
+	char filename[FILENAME_MAX], cpuext[64], *pfname;
+	int len;
 
-	if (!extra)
-		extra = "";
-	*cpu_suffix = '\0';
-
-	/* determine length of resulting filename, which looks like:
-	 *   <fileroot>[-<extra>][-cpu_<f>_<m>_<s>].exp
-	 */
-	n = strlen(p->y->fileroot);
-
-	if (*extra != '\0')
-		/* the extra string is prepended with a -.  */
-		n += 1 + strlen(extra);
+	memset(filename, 0, sizeof(filename));
+	memset(cpuext, 0, sizeof(cpuext));
 
 	if (p->conf->cpu.vendor != pcv_unknown) {
 		struct pt_cpu cpu;
-		int len;
 
 		cpu = p->conf->cpu;
 		if (cpu.stepping)
-			len = sprintf(cpu_suffix,
-				      "-cpu_%" PRIu16 "_%" PRIu8 "_%" PRIu8 "",
-				      cpu.family, cpu.model, cpu.stepping);
+			len = snprintf(cpuext, sizeof(cpuext),
+				       "-cpu_%" PRIu16 "_%" PRIu8 "_%" PRIu8,
+				       cpu.family, cpu.model, cpu.stepping);
 		else
-			len = sprintf(cpu_suffix,
-				      "-cpu_%" PRIu16 "_%" PRIu8 "", cpu.family,
-				      cpu.model);
+			len = snprintf(cpuext, sizeof(cpuext),
+				       "-cpu_%" PRIu16 "_%" PRIu8, cpu.family,
+				       cpu.model);
 
-		if (len < 0)
+		if ((len < 0) || (sizeof(cpuext) <= (size_t) len))
 			return NULL;
-
-		n += (size_t) len;
 	}
 
-	n += strlen(exp_suffix);
+	if (extra && *extra)
+		len = snprintf(filename, sizeof(filename), "%s-%s%s%s",
+			       p->y->fileroot, extra, cpuext, exp_suffix);
+	else
+		len = snprintf(filename, sizeof(filename), "%s-%s%s",
+			       p->y->fileroot, cpuext, exp_suffix);
 
-	/* trailing null character.  */
-	n += 1;
-
-	filename = malloc(n);
-	if (!filename)
+	if ((len < 0) || (sizeof(filename) <= (size_t) len))
 		return NULL;
 
-	strcpy(filename, p->y->fileroot);
-	if (*extra != '\0') {
-		strcat(filename, "-");
-		strcat(filename, extra);
-	}
-	strcat(filename, cpu_suffix);
-	strcat(filename, exp_suffix);
+	pfname = malloc((size_t) len + 1);
+	if (!pfname)
+		return NULL;
 
-	return filename;
+	pfname[len] = 0;
+
+	return memcpy(pfname, filename, (size_t) len);
 }
 
 /* Returns true if @c is part of a label; false otherwise.  */
@@ -312,11 +307,11 @@ static int islabelchar(int c)
 static int p_gen_expfile(struct parser *p)
 {
 	int errcode;
-	enum { slen = 1024 };
-	char s[slen];
+	char s[1024];
 	struct pt_directive *pd;
 	char *filename;
 	FILE *f;
+	size_t slen;
 
 	if (bug_on(!p))
 		return -err_internal;
@@ -342,19 +337,26 @@ static int p_gen_expfile(struct parser *p)
 
 	for (;;) {
 		int i;
-		char *line, *comment;
+		char *line, *comment, *end;
 
-		errcode = yasm_next_line(p->y, s, slen);
+		errcode = yasm_next_line(p->y, s, sizeof(s));
 		if (errcode < 0)
 			break;
 
+		slen = strnlen(s, sizeof(s));
+		if (sizeof(s) <= slen) {
+			errcode = -err_internal;
+			break;
+		}
+		end = &s[slen];
+
 		errcode = yasm_pd_parse(p->y, pd);
 		if (errcode < 0 && errcode != -err_no_directive)
 			break;
 
 		if (errcode == 0 && strcmp(pd->name, ".exp") == 0) {
 			fclose(f);
-			printf("%s\n", filename);
+			printf("%s ", filename);
 			free(filename);
 			filename = expfilename(p, pd->payload);
 			if (!filename)
@@ -374,12 +376,13 @@ static int p_gen_expfile(struct parser *p)
 		line += 1;
 
 		comment = strchr(line, '#');
-		if (comment)
+		if (comment) {
+			end = comment;
 			*comment = '\0';
-
+		}
 		/* remove trailing spaces.  */
-		for (i = (int) strlen(line)-1; i >= 0 && isspace(line[i]); i--)
-			line[i] = '\0';
+		for (end -= 1; line <= end && isspace(*end); --end)
+			*end = '\0';
 
 		for (;;) {
 			char *tmp, label[256];
@@ -442,7 +445,7 @@ static int p_gen_expfile(struct parser *p)
 			for (i = 0; islabelchar(line[i]); i++)
 				;
 
-			if (i > 255) {
+			if (l_max <= i) {
 				errcode = -err_label_name;
 				goto error;
 			}
@@ -546,11 +549,10 @@ error:
 
 	fclose(f);
 	if (errcode < 0 && errcode != -err_out_of_range) {
-		fprintf(stderr, "fatal: %s could not be created:\n", filename);
+		fprintf(stderr, "fatal: error generating %s:\n", filename);
 		yasm_print_err(p->y, "", errcode);
-		remove(filename);
 	} else
-		printf("%s\n", filename);
+		printf("%s ", filename);
 	free(filename);
 
 	/* If there are no lines left, we are done.  */
@@ -597,6 +599,143 @@ static int report_lib_error(struct parser *p, const char *message, int errcode)
 	return -err_pt_lib;
 }
 
+/* Parse any amount of whitespace from @pinput, including none.
+ *
+ * We want to stay within one source line so we only parse spaces and
+ * horizontal tabs.
+ *
+ * Returns zero on success and updates @input.
+ * Returns a negative integer on error.
+ */
+static int parse_whitespace(const char **pinput)
+{
+	const char *input;
+
+	if (!pinput)
+		return -err_internal;
+
+	input = *pinput;
+	if (!input)
+		return -err_internal;
+
+	while ((*input == ' ') || (*input == '\t'))
+		input += 1;
+
+	*pinput = input;
+	return 0;
+}
+
+/* Parse @size bytes of a literal @token in @input.
+ *
+ * Returns zero on success and updates @input.
+ * Returns a positive integer on failure.
+ * Returns a negative integer on error.
+ */
+static int parse_token_aux(const char **pinput, const char *token, size_t size)
+{
+	const char *input;
+	int status;
+
+	if (!pinput)
+		return -err_internal;
+
+	input = *pinput;
+	if (!input)
+		return -err_internal;
+
+	if (!token || !size)
+		return -err_internal;
+
+	status = parse_whitespace(&input);
+	if (status < 0)
+		return status;
+
+	if (strncmp(input, token, size) != 0)
+		return 1;
+
+	input += size;
+	*pinput = input;
+	return 0;
+}
+
+/* Parse a string literal @token in @input. */
+#define parse_token(input, token)				\
+	parse_token_aux(input, token, sizeof(token) - 1)
+
+/* Parse an @size-bit base-@base unsigned integer in @input.
+ *
+ * If an integer number can be parsed from @input, diagnoses exceeding the
+ * expected range with an error code return.
+ *
+ * Returns zero on success and updates @input.
+ * Returns a positive integer if @input does not start with an integer.
+ * Returns a negative integer on error.
+ */
+static int parse_uint(const char **pinput, unsigned long long *puint,
+		      uint8_t size, int base)
+{
+	unsigned long long uint;
+	const char *input;
+	char *end;
+
+	if (!pinput)
+		return -err_internal;
+
+	input = *pinput;
+	if (!input)
+		return -err_internal;
+
+	if (!puint || !size)
+		return -err_internal;
+
+	if (size > 64)
+		return -err_internal;
+
+	errno = 0;
+	uint = strtoull(input, &end, base);
+	if (input == end)
+		return 1;
+
+	if (errno == EINVAL)
+		return -err_internal;
+
+	if (errno == ERANGE)
+		return -err_parse_int_too_big;
+
+	if ((size < 64) && ((uint >> size) != 0))
+		return -err_parse_int_too_big;
+
+	*puint = uint;
+	*pinput = end;
+	return 0;
+}
+
+static int parse_uint_8(const char **pinput, uint8_t *uint)
+{
+	unsigned long long tmp;
+	int status;
+
+	status = parse_uint(pinput, &tmp, 8, 0);
+	if (status != 0)
+		return status;
+
+	*uint = (uint8_t) tmp;
+	return 0;
+}
+
+static int parse_uint_64(const char **pinput, uint64_t *uint)
+{
+	unsigned long long tmp;
+	int status;
+
+	status = parse_uint(pinput, &tmp, 64, 0);
+	if (status != 0)
+		return status;
+
+	*uint = (uint64_t) tmp;
+	return 0;
+}
+
 static int parse_mwait(uint32_t *hints, uint32_t *ext, char *payload)
 {
 	int errcode;
@@ -671,6 +810,273 @@ static int parse_c_state(uint8_t *state, uint8_t *sub_state, const char *input)
 	return 0;
 }
 
+/* Parse a mode.exec execution mode in @input into @packet.
+ *
+ * Returns zero on success and updates @input.
+ * Returns a positive integer on failure.
+ * Returns a negative integer on error.
+ */
+static int parse_exec_mode(const char **input,
+			   struct pt_packet_mode_exec *packet)
+{
+	int status;
+
+	if (!packet)
+		return -err_internal;
+
+	status = parse_token(input, "64bit");
+	if (status <= 0) {
+		if (status < 0)
+			return status;
+
+		packet->csl = 1;
+		packet->csd = 0;
+
+		return status;
+	}
+
+	status = parse_token(input, "32bit");
+	if (status <= 0) {
+		if (status < 0)
+			return status;
+
+		packet->csl = 0;
+		packet->csd = 1;
+
+		return status;
+	}
+
+	status = parse_token(input, "16bit");
+	if (status <= 0) {
+		if (status < 0)
+			return status;
+
+		packet->csl = 0;
+		packet->csd = 0;
+
+		return status;
+	}
+
+	return 1;
+}
+
+/* Parse mode.exec arguments in @input into @packet.
+ *
+ * Returns zero on success and updates @input.
+ * Returns a positive integer on failure.
+ * Returns a negative integer on error.
+ */
+static int parse_mode_exec(const char **input, const struct parser *p,
+			   struct pt_packet_mode_exec *packet)
+{
+	int status;
+
+	if (!p || !packet)
+		return -err_internal;
+
+	status = parse_exec_mode(input, packet);
+	if (status != 0) {
+		if (status < 0)
+			return status;
+
+		status = yasm_print_err(p->y, "mode.exec: bad argument, "
+					"expected \"16bit\", \"64bit\" or "
+					"\"32bit\"", -err_parse);
+		if (status < 0)
+			return status;
+
+		return 1;
+	}
+
+	status = parse_token(input, ",");
+	if (status != 0) {
+		if (status < 0)
+			return status;
+
+		packet->iflag = 0;
+		return 0;
+	}
+
+	status = parse_token(input, "if");
+	if (status != 0) {
+		if (status < 0)
+			return status;
+
+		status = yasm_print_err(p->y, "mode.exec: bad argument, "
+					"expected \"if\"", -err_parse);
+		if (status < 0)
+			return status;
+
+		return 1;
+	}
+
+	packet->iflag = 1;
+	return 0;
+}
+
+/* Parse cfe arguments in @input into @packet.
+ *
+ * Returns zero on success and updates @input.
+ * Returns a positive integer on failure.
+ * Returns a negative integer on error.
+ */
+static int parse_cfe(const char **input, const struct parser *p,
+		     struct pt_packet_cfe *packet)
+{
+	uint8_t type;
+	int status;
+
+	if (!p || !packet)
+		return -err_internal;
+
+	status = parse_uint_8(input, &type);
+	if (status != 0) {
+		if (status < 0)
+			return status;
+
+		status = yasm_print_err(p->y, "cfe: bad argument, expected "
+					"8-bit unsigned integer 'type'",
+					-err_parse);
+		if (status < 0)
+			return status;
+
+		return 1;
+	}
+
+	packet->type = (enum pt_cfe_type) type;
+
+	status = parse_token(input, ":");
+	if (status != 0) {
+		if (status < 0)
+			return status;
+
+		switch (packet->type) {
+		case pt_cfe_intr:
+		case pt_cfe_sipi:
+		case pt_cfe_vmexit_intr:
+		case pt_cfe_uintr:
+			status = yasm_print_err(p->y, "cfe: type needs "
+						"'vector' argument",
+						-err_parse);
+			if (status < 0)
+				return status;
+
+			return 1;
+
+		case pt_cfe_iret:
+		case pt_cfe_smi:
+		case pt_cfe_rsm:
+		case pt_cfe_init:
+		case pt_cfe_vmentry:
+		case pt_cfe_vmexit:
+		case pt_cfe_shutdown:
+		case pt_cfe_uiret:
+			packet->vector = 0;
+			break;
+		}
+	} else {
+		status = parse_uint_8(input, &packet->vector);
+		if (status != 0) {
+			if (status < 0)
+				return status;
+
+			status = yasm_print_err(p->y, "cfe: bad argument, "
+						"expected  8-bit unsigned "
+						"integer 'vector'", -err_parse);
+			if (status < 0)
+				return status;
+
+			return 1;
+		}
+	}
+
+	status = parse_token(input, ",");
+	if (status != 0) {
+		if (status < 0)
+			return status;
+
+		packet->ip = 0;
+		return 0;
+	}
+
+	status = parse_token(input, "ip");
+	if (status != 0) {
+		if (status < 0)
+			return status;
+
+		status = yasm_print_err(p->y, "cfe: bad argument, expected"
+					"'ip' keyword", -err_parse);
+		if (status < 0)
+			return status;
+
+		return 1;
+	}
+
+	packet->ip = 1;
+	return 0;
+}
+
+/* Parse evd arguments in @input into @packet.
+ *
+ * Returns zero on success and updates @input.
+ * Returns a positive integer on failure.
+ * Returns a negative integer on error.
+ */
+static int parse_evd(const char **input, const struct parser *p,
+		     struct pt_packet_evd *packet)
+{
+	uint8_t type;
+	int status;
+
+	if (!p || !packet)
+		return -err_internal;
+
+	status = parse_uint_8(input, &type);
+	if (status != 0) {
+		if (status < 0)
+			return status;
+
+		status = yasm_print_err(p->y, "evd: bad argument, expected "
+					"8-bit unsigned integer 'type'",
+					-err_parse);
+		if (status < 0)
+			return status;
+
+		return 1;
+	}
+
+	packet->type = (enum pt_evd_type) type;
+
+	status = parse_token(input, ":");
+	if (status != 0) {
+		if (status < 0)
+			return status;
+
+		status = yasm_print_err(p->y, "evd: bad argument, expected "
+					"':' separator", -err_parse);
+		if (status < 0)
+			return status;
+
+		return 1;
+	}
+
+	status = parse_uint_64(input, &packet->payload);
+	if (status != 0) {
+		if (status < 0)
+			return status;
+
+		status = yasm_print_err(p->y, "evd: bad argument, expected "
+					"64-bit unsigned integer 'payload'",
+					-err_parse);
+		if (status < 0)
+			return status;
+
+		return 1;
+	}
+
+	return 0;
+}
+
 static int pt_raw(struct parser *p, struct pt_encoder *e,
 		  const void *buffer, size_t size)
 {
@@ -851,21 +1257,22 @@ static int p_process_pt(struct parser *p, struct pt_encoder *e)
 		}
 		packet.type = ppt_fup;
 	} else if (strcmp(directive, "mode.exec") == 0) {
-		if (strcmp(payload, "16bit") == 0) {
-			packet.payload.mode.bits.exec.csl = 0;
-			packet.payload.mode.bits.exec.csd = 0;
-		} else if (strcmp(payload, "64bit") == 0) {
-			packet.payload.mode.bits.exec.csl = 1;
-			packet.payload.mode.bits.exec.csd = 0;
-		} else if (strcmp(payload, "32bit") == 0) {
-			packet.payload.mode.bits.exec.csl = 0;
-			packet.payload.mode.bits.exec.csd = 1;
-		} else {
-			errcode = yasm_print_err(p->y,
-						 "mode.exec: argument must be one of \"16bit\", \"64bit\" or \"32bit\"",
-						 -err_parse);
+		const char *cpl;
+
+		cpl = (const char *) payload;
+		errcode = parse_mode_exec(&cpl, p,
+					  &packet.payload.mode.bits.exec);
+		if (errcode != 0) {
+			if (errcode < 0)
+				yasm_print_err(p->y,
+					       "mode.exec: parsing failed",
+					       errcode);
+			else
+				errcode = -err_parse;
+
 			return errcode;
 		}
+
 		packet.payload.mode.leaf = pt_mol_exec;
 		packet.type = ppt_mode;
 	} else if (strcmp(directive, "mode.tsx") == 0) {
@@ -1088,6 +1495,38 @@ static int p_process_pt(struct parser *p, struct pt_encoder *e)
 
 			packet.payload.ptw.ip = 1;
 		}
+	} else if (strcmp(directive, "cfe") == 0) {
+		const char *cpl;
+
+		cpl = (const char *) payload;
+		errcode = parse_cfe(&cpl, p, &packet.payload.cfe);
+		if (errcode != 0) {
+			if (errcode < 0)
+				yasm_print_err(p->y, "cfe: parsing failed",
+					       errcode);
+			else
+				errcode = -err_parse;
+
+			return errcode;
+		}
+
+		packet.type = ppt_cfe;
+	} else if (strcmp(directive, "evd") == 0) {
+		const char *cpl;
+
+		cpl = (const char *) payload;
+		errcode = parse_evd(&cpl, p, &packet.payload.evd);
+		if (errcode != 0) {
+			if (errcode < 0)
+				yasm_print_err(p->y, "evd: parsing failed",
+					       errcode);
+			else
+				errcode = -err_parse;
+
+			return errcode;
+		}
+
+		packet.type = ppt_evd;
 	} else if (strcmp(directive, "raw-8") == 0) {
 		uint8_t value;
 
@@ -1158,6 +1597,7 @@ static int sb_open(struct parser *p, const char *fmt, const char *src,
 	const char *root;
 	char name[FILENAME_MAX];
 	FILE *file;
+	int errcode;
 
 	if (bug_on(!p) || bug_on(!p->y) || bug_on(!prio))
 		return -err_internal;
@@ -1196,12 +1636,13 @@ static int sb_open(struct parser *p, const char *fmt, const char *src,
 
 		memset(&sbfiles->sbfile, 0, sizeof(sbfiles->sbfile));
 
-		sbfiles->sbfile.name = duplicate_str(name);
-		if (!sbfiles->sbfile.name) {
-			yasm_print_err(p->y, "open", -err_no_mem);
+		errcode = duplicate_name(&sbfiles->sbfile.name, name,
+					 FILENAME_MAX);
+		if (errcode < 0) {
+			yasm_print_err(p->y, "open", errcode);
 			fclose(file);
 			free(sbfiles);
-			return -err_no_mem;
+			return errcode;
 		}
 
 		sbfiles->sbfile.file = file;
@@ -1745,7 +2186,8 @@ static int pevent_comm(struct parser *p, const char *pid, const char *tid,
 		uint8_t buffer[FILENAME_MAX];
 	} record;
 	struct pev_event event;
-	int errcode;
+	size_t limit;
+	int errcode, len;
 
 	if (bug_on(!p) || bug_on(!p->y))
 		return -err_internal;
@@ -1765,7 +2207,12 @@ static int pevent_comm(struct parser *p, const char *pid, const char *tid,
 		return errcode;
 	}
 
-	strcpy(record.comm.comm, comm);
+	limit = sizeof(record.buffer) - sizeof(record.comm);
+	len = snprintf(record.comm.comm, limit, "%s", comm);
+	if (len < 0)
+		return -err_parse;
+	if (limit <= (size_t) len)
+		return -err_name_too_long;
 
 	event.type = PERF_RECORD_COMM;
 	event.misc = misc;
@@ -2469,8 +2916,9 @@ static int p_process_sb(struct parser *p)
  */
 static int p_process(struct parser *p, struct pt_encoder *e)
 {
-	char *directive, *tmp;
+	char *directive, *tmp, *end;
 	struct pt_directive *pd;
+	size_t nlen;
 
 	if (bug_on(!p))
 		return -err_internal;
@@ -2481,6 +2929,13 @@ static int p_process(struct parser *p, struct pt_encoder *e)
 
 	directive = pd->name;
 
+	nlen = strnlen(directive, pd_len);
+	if (pd_len <= nlen)
+		return -err_internal;
+
+	/* Plus 1 for termination. */
+	end = directive + nlen + 1;
+
 	/* We must have a directive. */
 	if (!directive || (strcmp(directive, "") == 0))
 		return yasm_print_err(p->y, "invalid syntax",
@@ -2507,7 +2962,7 @@ static int p_process(struct parser *p, struct pt_encoder *e)
 		char *pt_label_name;
 		uint64_t x;
 		int errcode, bytes_written;
-		size_t len;
+		size_t limit, len;
 
 		pt_label_name = directive;
 		directive = tmp+1;
@@ -2572,8 +3027,16 @@ static int p_process(struct parser *p, struct pt_encoder *e)
 			return errcode;
 
 		/* Update the directive name in the parser. */
-		len = strlen(directive) + 1;
+		if (end <= directive)
+			return -err_internal;
+
+		limit = (size_t) ((uintptr_t) end - (uintptr_t) directive);
+		len = strnlen(directive, limit);
+		if (limit <= len)
+			return -err_internal;
+
 		memmove(pd->name, directive, len);
+		pd->name[len] = '\0';
 	}
 
 	switch (pd->kind) {
