@@ -1,5 +1,5 @@
 /* windmc.c -- a program to compile Windows message files.
-   Copyright (C) 2007-2022 Free Software Foundation, Inc.
+   Copyright (C) 2007-2023 Free Software Foundation, Inc.
    Written by Kai Tietz, Onevision.
 
    This file is part of GNU Binutils.
@@ -746,12 +746,14 @@ write_bin (void)
       nd = convert_unicode_to_ACP (n->lang->sval);
 
       /* Prepare filename for binary output.  */
-      filename = xmalloc (strlen (nd) + 4 + 1 + strlen (mcset_mc_basename) + 1 + strlen (mcset_rc_dir));
-      strcpy (filename, mcset_rc_dir);
+      filename = xmalloc (strlen (nd) + 4 + 1 + strlen (mcset_mc_basename)
+			  + 1 + strlen (mcset_rc_dir));
+      char *out = filename;
+      out = stpcpy (out, mcset_rc_dir);
       if (mcset_prefix_bin)
-	sprintf (filename + strlen (filename), "%s_", mcset_mc_basename);
-      strcat (filename, nd);
-      strcat (filename, ".bin");
+	out += sprintf (out, "%s_", mcset_mc_basename);
+      out = stpcpy (out, nd);
+      out = stpcpy (out, ".bin");
 
       /* Write message file.  */
       windmc_write_bin (filename, &mc_nodes_lang[i], (c - i));
